package com.jdnevnik.ainterface;

import com.jdnevnik.ainterface.logging.*;

import java.awt.*;
import java.util.*;
import java.util.concurrent.*;

public class Automata extends Thread {
    protected Map<String, com.jdnevnik.ainterface.State> _states =
            new HashMap<String, com.jdnevnik.ainterface.State>();
    protected Map<String, Transition> _transitions = new HashMap<String, Transition>();
    protected Map<String, Event> _events = new HashMap<String, Event>();
    private Component _rootComponent = null;

    private ConcurrentLinkedQueue<Event> _eventsQueue = new ConcurrentLinkedQueue<Event>();

    private com.jdnevnik.ainterface.State _currentState = null;
    private Event _processingEvent = null;

    private Map<String, String> _attributes = new HashMap<String, String>();

    private static final String MONITOR = "Monitor";

    public static final ILogger LOGGER = new SimpleLogger();

    public void removeState(State aState) {
        _states.remove(aState);
    }

    public Component getRootComponent() {
        return _rootComponent;
    }

    public void setRootComponent(Component aRootComponent) {
        _rootComponent = aRootComponent;
    }

    public void processEvent(Event event) {
        _eventsQueue.add(event);
        synchronized (MONITOR) {
            MONITOR.notifyAll();
        }
    }

    public void run() {
        while (true) {
            if (_eventsQueue.size() > 0) {
                _processingEvent = _eventsQueue.poll();

                _currentState = _processingEvent.getState();
                startAutomata();
                _processingEvent = null;
            }

            try {
                synchronized (MONITOR) {
                    MONITOR.wait();
                }
            } catch (InterruptedException e) {
                e.printStackTrace();
            }
        }
    }

    private void startAutomata() {
        while (_currentState != null) {
            _currentState.processActions(this);
            Transition transition = _currentState.getTransition(this);
            if (transition != null) {
                transition.activate(this);
            } else {
                _currentState = null;
            }
        }
    }

    public com.jdnevnik.ainterface.State getCurrentState() {
        return _currentState;
    }

    public void setCurrentState(com.jdnevnik.ainterface.State aCurrentState) {
        _currentState = aCurrentState;
    }

    public void setAttribute(String aName, Object aAttribute) {
        _attributes.put(aName, aAttribute.toString());
    }

    private String getAttribute(String aName) {
        if (_attributes.containsKey(aName)) {
            return _attributes.get(aName);
        } else {
            return "";
        }
    }

    public String getAttributeAsString(String aName) {
        return getAttribute(aName);
    }

    public int getAttributeAsInt(String aName) {
        return Integer.parseInt(getAttribute(aName));
    }

    public double getAttributeAsDouble(String aName) {
        return Double.parseDouble(getAttribute(aName));
    }

    public Event getProcessingEvent() {
        return _processingEvent;
    }
}
