package com.jdnevnik.ainterface;

import com.jdnevnik.ainterface.listeners.*;
import org.dom4j.*;
import org.dom4j.io.*;

import javax.swing.*;

import java.awt.*;
import java.io.*;
import java.util.*;

public class XMLAutomata extends Automata {
    private static final String ELEMENT_ACTIONS = "actions";
    private static final String ELEMENT_EVENT = "event";
    private static final String ELEMENT_COMPONENT = "component";
    private static final String ELEMENT_CLASS = "class";
    private static final String ELEMENT_TYPE = "type";
    private static final String ELEMENT_STATE = "state";
    private static final String ELEMENT_CONDITION = "condition";

    private static final String ATTRIBUTE_NAME = "name";
    private static final String ATTRIBUTE_START_STATE_NAME = "startStateName";
    private static final String ATTRIBUTE_TRANSITION = "transition";
    private static final String ATTRIBUTE_TARGET_STATE_NAME = "targetStateName";

    public XMLAutomata(String aXMLFile) throws DocumentException, FileNotFoundException {
        loadFromXML(aXMLFile);
    }

    private void loadFromXML(String aFile) throws FileNotFoundException, DocumentException {
        Document document = new SAXReader().read(new FileInputStream(aFile));
        Element root = document.getRootElement();

        try {
            setRootComponent((JFrame) Class.forName(root.element("rootComponent").getStringValue()).newInstance());
        } catch (Exception e) {
            e.printStackTrace();
        }

        HashMap<String, String> transitionTargets = new HashMap<String, String>();

        Iterator transitions = root.elementIterator(ATTRIBUTE_TRANSITION);
        while (transitions.hasNext()) {
            Element element = (Element) transitions.next();
            String name = element.attribute(ATTRIBUTE_NAME).getValue();

            transitionTargets.put(name, element.attribute(ATTRIBUTE_TARGET_STATE_NAME).getValue());

            Transition transition = new Transition(name);
            transition.setActions(getActionsFromAttribute(element));
            _transitions.put(name, transition);
        }

        Iterator states = root.elementIterator(ELEMENT_STATE);
        while (states.hasNext()) {
            Element element = (Element) states.next();
            String name = element.attribute(ATTRIBUTE_NAME).getValue();

            com.jdnevnik.ainterface.State state = new com.jdnevnik.ainterface.State(name);
            state.setActions(getActionsFromAttribute(element));

            Iterator conditions = element.elementIterator(ELEMENT_CONDITION);
            while (conditions.hasNext()) {
                Element jelement = (Element) conditions.next();
                String transition = jelement.attribute(ATTRIBUTE_TRANSITION).getValue();
                String className = jelement.attribute(ELEMENT_CLASS).getValue();

                try {
                    Class clazz = Class.forName(className);
                    ICondition condition = (ICondition) clazz.getConstructor().newInstance();
                    state.addTransition(condition, _transitions.get(transition));
                } catch (Exception e) {
                    e.printStackTrace();
                }
            }

            _states.put(name, state);
        }

        Iterator events = root.elementIterator(ELEMENT_EVENT);
        while (events.hasNext()) {
            Element element = (Element) events.next();
            String componentName = element.attribute(ELEMENT_COMPONENT).getValue();
            String className = element.attribute(ELEMENT_CLASS).getValue();
            Attribute attributeType = element.attribute(ELEMENT_TYPE);
            String type = attributeType == null ? "" : attributeType.getValue();
            String startState = element.attribute(ATTRIBUTE_START_STATE_NAME).getValue();

            try {
                Component component = (Component) getRootComponent().getClass().
                        getMethod("get" + upperCaseFirstLetter(componentName)).
                        invoke(getRootComponent());

                Class clazz = Class.forName("com.jdnevnik.ainterface.listeners.Auto" + upperCaseFirstLetter(className) + "Listener");
                Listener listener = (Listener) clazz.getConstructor(Automata.class, com.jdnevnik.ainterface.State.class).newInstance(this, _states.get(startState));
                listener.addListenerToComponent(component, type);
            } catch (Exception e) {
                e.printStackTrace();
            }
        }

        for (Iterator<Map.Entry<String, String>> iterator = transitionTargets.entrySet().iterator(); iterator.hasNext();) {
            Map.Entry<String, String> entry = iterator.next();
            _transitions.get(entry.getKey()).setTargetState(_states.get(entry.getValue()));
        }
    }

    private static IActions getActionsFromAttribute(Element aElement) {
        IActions actions = null;

        if (aElement.attribute(ELEMENT_ACTIONS) != null) {
            try {
                Class clazz = Class.forName(aElement.attribute(ELEMENT_ACTIONS).getValue());
                actions = (IActions) clazz.getConstructor().newInstance();
            } catch (Exception e) {
                e.printStackTrace();
            }
        }
        return actions;
    }

    private static String upperCaseFirstLetter(String aComponentName) {
        return String.valueOf(Character.toUpperCase(aComponentName.charAt(0))) + aComponentName.substring(1);
    }
}
